import React, { PureComponent } from 'react';
import { Form, Input, Row, Col, Tabs, Icon, Collapse, Button, Select, message } from 'antd';
import Schema from "async-validator";
import { FormButtonModel, FormPageModel, FormItemModel, FormDRUIPartModel, FormPartModel, FormGroupPanelModel, FormIFrameModel, FormRowItemModel, FormTabPageModel, FormTabPanelModel, FormUserControlModel } from '../../../ibizsys/form-detail';
import { Util } from '../../../ibizsys/utils/util';

import AppDropDownList  from '../../../ibizsys/components/app-dropdown-list/app-dropdown-list';
import AppRadioGroup from '../../../ibizsys/components/app-radio-group/app-radio-group';
import AppCheckBox from '../../../ibizsys/components/app-checkbox/app-checkbox';
import AppCheckBoxGroup from '../../../ibizsys/components/app-checkbox-group/app-checkbox-group';
import AppDatePicker from '../../../ibizsys/components/app-date-picker/app-date-picker';
import AppTimePicker from '../../../ibizsys/components/app-time-picker/app-time-picker';
import AppImageUpload from '../../../ibizsys/components/app-image-upload/app-image-upload';
import AppFileUpload from '../../../ibizsys/components/app-file-upload/app-file-upload';
import AppRichTextEditor from '../../../ibizsys/components/app-rich-text-editor/app-rich-text-editor';

import './main.less';
const { TabPane } = Tabs;
const { Panel } = Collapse;
const { Option } = Select;
const { TextArea  } = Input;

/**
 * 表单属性
 *
 * @export
 * @interface MainFormData
 */
export class MainFormData {
    /**
     * 实体3名称
     *
     * @type {*}
     * @memberof MainFormData
     */
    entity3name?: any;
    /**
     * 
     *
     * @type {*}
     * @memberof MainFormData
     */
    formitem?: any;
    /**
     * 
     *
     * @type {*}
     * @memberof MainFormData
     */
    formitem2?: any;
    /**
     * 
     *
     * @type {*}
     * @memberof MainFormData
     */
    formitem1?: any;
    /**
     * 
     *
     * @type {*}
     * @memberof MainFormData
     */
    formitem3?: any;
    /**
     * 
     *
     * @type {*}
     * @memberof MainFormData
     */
    formitem4?: any;
    /**
     * 实体3标识
     *
     * @type {*}
     * @memberof MainFormData
     */
    entity3id?: any;
}

/**
 * 输入属性
 *
 * @export
 * @interface MainProps
 */
export interface MainProps {
    /**
     * 表单数据对象
     *
     * @type {MainFormData}
     * @memberof MainProps
     */
    data?: MainFormData;
    /**
     * 表单分页切换
     *
     * @memberof MainProps
     */
    tabChange?: (key: string) => void;
    /**
     * 表单项之变化
     *
     * @memberof MainProps
     */
    formItemChange?:(key:any) => void;
}

/**
 * Main
 *
 * @export
 * @class Main
 * @extends {PureComponent}
 */
export default class Main extends PureComponent<MainProps> {
    /**
     * 表单数据
     *
     * @type {*}
     * @memberof Main
     */
    public data: MainFormData = new MainFormData();

    /**
     * 初始化实例
     *
     * @type {*}
     * @memberof Main
     */
    constructor(props) {
        super(props);
        if (this.data && this.props.data && !Object.is(JSON.stringify(this.data), JSON.stringify(this.props.data))) {
            Object.assign(this.data, this.props.data);
        }
        this.initForm();
    }
    
    /**
     * 在组件接收到一个新的 prop (或更新后)时被调用
     *
     * @memberof IBizViewController
     */
    public componentWillReceiveProps(nextProps: Readonly<MainProps>, nextContext: any) {
        if (this.data && nextProps.data && !Object.is(JSON.stringify(this.data), JSON.stringify(nextProps.data))) {
            Object.assign(this.data, nextProps.data);
        }
    }

    /**
     * 在组件从 DOM 中移除的时候立刻被调用
     *
     * @memberof IBizViewController
     */
    public componentWillUnmount() {

    }

    /**
     * 属性值规则
     *
     * @type {*}
     * @memberof Main
     */
    public rules: any = {
        srfupdatedate: [
            { type: 'string', message: '更新时间 值必须为字符串类型', trigger: 'change' },
            { required: false, type: 'string', message: '更新时间 值不能为空', trigger: 'change' },
            { required: false, type: 'string', message: '更新时间 值不能为空', trigger: 'blur' },
        ],
        srforikey: [
            { type: 'string', message: ' 值必须为字符串类型', trigger: 'change' },
            { required: false, type: 'string', message: ' 值不能为空', trigger: 'change' },
            { required: false, type: 'string', message: ' 值不能为空', trigger: 'blur' },
        ],
        srfkey: [
            { type: 'string', message: '实体3标识 值必须为字符串类型', trigger: 'change' },
            { required: false, type: 'string', message: '实体3标识 值不能为空', trigger: 'change' },
            { required: false, type: 'string', message: '实体3标识 值不能为空', trigger: 'blur' },
        ],
        srfmajortext: [
            { type: 'string', message: '实体3名称 值必须为字符串类型', trigger: 'change' },
            { required: false, type: 'string', message: '实体3名称 值不能为空', trigger: 'change' },
            { required: false, type: 'string', message: '实体3名称 值不能为空', trigger: 'blur' },
        ],
        srftempmode: [
            { type: 'string', message: ' 值必须为字符串类型', trigger: 'change' },
            { required: false, type: 'string', message: ' 值不能为空', trigger: 'change' },
            { required: false, type: 'string', message: ' 值不能为空', trigger: 'blur' },
        ],
        srfuf: [
            { type: 'string', message: ' 值必须为字符串类型', trigger: 'change' },
            { required: false, type: 'string', message: ' 值不能为空', trigger: 'change' },
            { required: false, type: 'string', message: ' 值不能为空', trigger: 'blur' },
        ],
        srfdeid: [
            { type: 'string', message: ' 值必须为字符串类型', trigger: 'change' },
            { required: false, type: 'string', message: ' 值不能为空', trigger: 'change' },
            { required: false, type: 'string', message: ' 值不能为空', trigger: 'blur' },
        ],
        srfsourcekey: [
            { type: 'string', message: ' 值必须为字符串类型', trigger: 'change' },
            { required: false, type: 'string', message: ' 值不能为空', trigger: 'change' },
            { required: false, type: 'string', message: ' 值不能为空', trigger: 'blur' },
        ],
        entity3name: [
            { type: 'string', message: '实体3名称 值必须为字符串类型', trigger: 'change' },
            { required: false, type: 'string', message: '实体3名称 值不能为空', trigger: 'change' },
            { required: false, type: 'string', message: '实体3名称 值不能为空', trigger: 'blur' },
        ],
        formitem: [
            { type: 'string', message: ' 值必须为字符串类型', trigger: 'change' },
            { required: false, type: 'string', message: ' 值不能为空', trigger: 'change' },
            { required: false, type: 'string', message: ' 值不能为空', trigger: 'blur' },
        ],
        formitem2: [
            { type: 'string', message: ' 值必须为字符串类型', trigger: 'change' },
            { required: false, type: 'string', message: ' 值不能为空', trigger: 'change' },
            { required: false, type: 'string', message: ' 值不能为空', trigger: 'blur' },
        ],
        formitem1: [
            { type: 'string', message: ' 值必须为字符串类型', trigger: 'change' },
            { required: false, type: 'string', message: ' 值不能为空', trigger: 'change' },
            { required: false, type: 'string', message: ' 值不能为空', trigger: 'blur' },
        ],
        formitem3: [
            { type: 'string', message: ' 值必须为字符串类型', trigger: 'change' },
            { required: false, type: 'string', message: ' 值不能为空', trigger: 'change' },
            { required: false, type: 'string', message: ' 值不能为空', trigger: 'blur' },
        ],
        formitem4: [
            { type: 'string', message: ' 值必须为字符串类型', trigger: 'change' },
            { required: false, type: 'string', message: ' 值不能为空', trigger: 'change' },
            { required: false, type: 'string', message: ' 值不能为空', trigger: 'blur' },
        ],
        entity3id: [
            { type: 'string', message: '实体3标识 值必须为字符串类型', trigger: 'change' },
            { required: false, type: 'string', message: '实体3标识 值不能为空', trigger: 'change' },
            { required: false, type: 'string', message: '实体3标识 值不能为空', trigger: 'blur' },
        ],
    }

    /**
     * 详情模型集合
     *
     * @type {*}
     * @memberof Main
     */
    public detailsModel: any = {
        group1: new FormGroupPanelModel({ caption: '实体3基本信息', detailType: 'GROUPPANEL', name: 'group1', visible: true, isShowCaption: true, form: this, uiActionGroup: { caption: '', langbase: '', extractMode: 'ITEM', details: [] } })
,
        grouppanel2: new FormGroupPanelModel({ caption: '分组面板', detailType: 'GROUPPANEL', name: 'grouppanel2', visible: true, isShowCaption: false, form: this, uiActionGroup: { caption: '', langbase: '', extractMode: 'ITEM', details: [] } })
,
        grouppanel4: new FormGroupPanelModel({ caption: '分组面板', detailType: 'GROUPPANEL', name: 'grouppanel4', visible: true, isShowCaption: false, form: this, uiActionGroup: { caption: '', langbase: '', extractMode: 'ITEM', details: [] } })
,
        grouppanel1: new FormGroupPanelModel({ caption: '分组面板', detailType: 'GROUPPANEL', name: 'grouppanel1', visible: true, isShowCaption: false, form: this, uiActionGroup: { caption: '', langbase: '', extractMode: 'ITEM', details: [] } })
,
        tabpanel1: new FormTabPanelModel({ caption: '', detailType: 'TABPANEL', name: 'tabpanel1', visible: true, isShowCaption: false, form: this, tabPages: [] })
,
        formpage1: new FormPageModel({ caption: '基本信息', detailType: 'FORMPAGE', name: 'formpage1', visible: true, isShowCaption: true, form: this })
,
        srfupdatedate: new FormItemModel({ caption: '更新时间', detailType: 'FORMITEM', name: 'srfupdatedate', visible: true, isShowCaption: true, form: this, disabled: false,error:"",isEmptyCaption:"false",validateStatus:"",required:false,enableCond: 3 })
,
        srforikey: new FormItemModel({ caption: '', detailType: 'FORMITEM', name: 'srforikey', visible: true, isShowCaption: true, form: this, disabled: false,error:"",isEmptyCaption:"false",validateStatus:"",required:false,enableCond: 3 })
,
        srfkey: new FormItemModel({ caption: '实体3标识', detailType: 'FORMITEM', name: 'srfkey', visible: true, isShowCaption: true, form: this, disabled: false,error:"",isEmptyCaption:"false",validateStatus:"",required:false,enableCond: 3 })
,
        srfmajortext: new FormItemModel({ caption: '实体3名称', detailType: 'FORMITEM', name: 'srfmajortext', visible: true, isShowCaption: true, form: this, disabled: false,error:"",isEmptyCaption:"false",validateStatus:"",required:false,enableCond: 3 })
,
        srftempmode: new FormItemModel({ caption: '', detailType: 'FORMITEM', name: 'srftempmode', visible: true, isShowCaption: true, form: this, disabled: false,error:"",isEmptyCaption:"false",validateStatus:"",required:false,enableCond: 3 })
,
        srfuf: new FormItemModel({ caption: '', detailType: 'FORMITEM', name: 'srfuf', visible: true, isShowCaption: true, form: this, disabled: false,error:"",isEmptyCaption:"false",validateStatus:"",required:false,enableCond: 3 })
,
        srfdeid: new FormItemModel({ caption: '', detailType: 'FORMITEM', name: 'srfdeid', visible: true, isShowCaption: true, form: this, disabled: false,error:"",isEmptyCaption:"false",validateStatus:"",required:false,enableCond: 3 })
,
        srfsourcekey: new FormItemModel({ caption: '', detailType: 'FORMITEM', name: 'srfsourcekey', visible: true, isShowCaption: true, form: this, disabled: false,error:"",isEmptyCaption:"false",validateStatus:"",required:false,enableCond: 3 })
,
        entity3name: new FormItemModel({ caption: '实体3名称', detailType: 'FORMITEM', name: 'entity3name', visible: true, isShowCaption: true, form: this, disabled: false,error:"",isEmptyCaption:"false",validateStatus:"",required:false,enableCond: 3 })
,
        formitem: new FormItemModel({ caption: '', detailType: 'FORMITEM', name: 'formitem', visible: true, isShowCaption: true, form: this, disabled: false,error:"",isEmptyCaption:"false",validateStatus:"",required:false,enableCond: 3 })
,
        formitem2: new FormItemModel({ caption: '', detailType: 'FORMITEM', name: 'formitem2', visible: true, isShowCaption: true, form: this, disabled: false,error:"",isEmptyCaption:"false",validateStatus:"",required:false,enableCond: 3 })
,
        formitem1: new FormItemModel({ caption: '', detailType: 'FORMITEM', name: 'formitem1', visible: true, isShowCaption: true, form: this, disabled: false,error:"",isEmptyCaption:"false",validateStatus:"",required:false,enableCond: 3 })
,
        formitem3: new FormItemModel({ caption: '', detailType: 'FORMITEM', name: 'formitem3', visible: true, isShowCaption: true, form: this, disabled: false,error:"",isEmptyCaption:"false",validateStatus:"",required:false,enableCond: 3 })
,
        formitem4: new FormItemModel({ caption: '', detailType: 'FORMITEM', name: 'formitem4', visible: true, isShowCaption: true, form: this, disabled: false,error:"",isEmptyCaption:"false",validateStatus:"",required:false,enableCond: 3 })
,
        entity3id: new FormItemModel({ caption: '实体3标识', detailType: 'FORMITEM', name: 'entity3id', visible: true, isShowCaption: true, form: this, disabled: false,error:"",isEmptyCaption:"false",validateStatus:"",required:false,enableCond: 3 })
,
    }

    /**
      * 代码表模型
      *
      * @type {*}
      * @memberof Main
      */      
    public codelistModel:any = {
      }

    /**
     * 表单逻辑
     *
     * @private
     * @param { name, newVal, oldVal }
     * @memberof Main
     */
    private formLogic(name: string, newVal: any, oldVal?: any): void {
                





















    }


    /**
     * 表单分页切换
     *
     * @param {string} key 分页标识
     * @memberof Main
     */
    public tabChange(key: string): void {
        if (this.props.tabChange) {
            this.props.tabChange(key);
        }
    }

    /**
     * 表单值变化事件
     *
     * @memberof Main
     */
    public formItemChange(item: string, value: any, isFirst?: boolean) {
        this.data[item] = value;
        this.formLogic(item, value);
        if (!isFirst) {
            this.validateItem(item, value);
            if (this.props.formItemChange) {
                this.props.formItemChange({name:item,value:value});
            }
        }
        this.tick();
    }

    /**
     * 按钮点击事件
     *
     * @memberof Main
     */
    public buttonClick($event:any,value:String){
        message.info('点击了'+value);
    }

    /**
     * 初始化表单
     * 
     * @memberof Main
     */
    public initForm() {
        if (Object.keys(this.data).length > 0) {
            Object.keys(this.data).forEach((item) => {
                this.formItemChange(item, this.data[item], true);
            })
        }
    }

    /**
     * 校验单个表单项
     * @param item 
     * @param value 
     */
    public validateItem(item: string, value: any) {
        // 做校验
        // 1.获取数值和规则
        const rule = this.rules[item];
        // 2.创建校验规则
        const schema = new Schema({ [item]: rule })
        // 校验返回Promise
        return schema.validate({ [item]: value },undefined,errors => {
            if (errors) {
                // 有错
                this.detailsModel[item].setError(errors[0].message);
            } else {
                this.detailsModel[item].setError('');
            }
        })
    }

    /**
     * 校验表单
     * 
     * @memberof Main
     */
    public validateForm(callback:Function) {
        let tasks: any;
        if (Object.keys(this.data).length > 0) {
            tasks = Object.keys(this.data).map((item) => {
                this.validateItem(item, this.data[item]);
            })
        }
        Promise.all(tasks)
            .then(() => callback(true))
            .catch(() => callback(false))
    }

    /**
     * 绘制内容
     *
     * @returns
     * @memberof Main
     */
    public render(): any {
        return <>
            <div className="ibiz-form main">
                <Form layout="inline">
                {this.detailsModel.group1.visible?<Col style={{}}  lg={{ span: 24, offset: 0}}>
        <Collapse defaultActiveKey={['group1']} className=''>
    <Panel header="实体3基本信息" key="group1">
        <Row>
                
        {this.detailsModel.entity3name.visible?<Col style={{}}  lg={{ span: 24, offset: 0}}>
                <div className={`app-form-item app-form-item-left`}>
            {this.detailsModel.entity3name.isShowCaption && !this.detailsModel.entity3name.isEmptyCaption ?
                <div className="app-form-item-label" style={{ width: '130px' }}>
                    {this.detailsModel.entity3name.required ? <span className="isrequired">*</span> : ""}实体3名称
                </div> : ""}
            <div className="app-form-item-container" style={{ width: "calc(100% - 130px)" }}>
                <Form.Item    key={'entity3name'}  validateStatus={this.detailsModel.entity3name.validateStatus} help={this.detailsModel.entity3name.error}>
                <Input id={'entity3name'} disabled={this.detailsModel.entity3name.disabled} value={this.data.entity3name}     onChange={($event) =>{this.formItemChange('entity3name',$event.target.value)}}/>
                </Form.Item>
            </div>
        </div> 
        
            </Col>:""}
        
            </Row>
    </Panel>
</Collapse>
    </Col>:""}
    {this.detailsModel.formitem.visible?<Col style={{}}  lg={{ span: 24, offset: 0}}>
        <div className={`app-form-item app-form-item-left`}>
    {this.detailsModel.formitem.isShowCaption && !this.detailsModel.formitem.isEmptyCaption ?
        <div className="app-form-item-label" style={{ width: '130px' }}>
            {this.detailsModel.formitem.required ? <span className="isrequired">*</span> : ""}
        </div> : ""}
    <div className="app-form-item-container" style={{ width: "calc(100% - 130px)" }}>
        <Form.Item    key={'formitem'}  validateStatus={this.detailsModel.formitem.validateStatus} help={this.detailsModel.formitem.error}>
        <Input id={'formitem'} disabled={this.detailsModel.formitem.disabled} value={this.data.formitem}     onChange={($event) =>{this.formItemChange('formitem',$event.target.value)}}/>
        </Form.Item>
    </div>
</div> 

    </Col>:""}
    {this.detailsModel.grouppanel1.visible?<Col style={{}}  lg={{ span: 24, offset: 0}}>
        <Collapse defaultActiveKey={['grouppanel1']} className=''>
    <Panel header="分组面板" key="grouppanel1">
        <Row>
                
        {this.detailsModel.grouppanel2.visible?<Col style={{}}  sm={{ span: 8, offset: 0 }} md={{span: 8, offset: 0}} lg={{ span: 8, offset: 0}} xl={{ span: 8, offset: 0}}>
                <Collapse defaultActiveKey={['grouppanel2']} className=''>
            <Panel header="分组面板" key="grouppanel2">
                <Row>
                        
                
                    </Row>
            </Panel>
        </Collapse>
            </Col>:""}
            {this.detailsModel.grouppanel4.visible?<Col style={{}}  sm={{ span: 8, offset: 0 }} md={{span: 8, offset: 0}} lg={{ span: 8, offset: 0}} xl={{ span: 8, offset: 0}}>
                <Collapse defaultActiveKey={['grouppanel4']} className=''>
            <Panel header="分组面板" key="grouppanel4">
                <Row>
                        
                
                    </Row>
            </Panel>
        </Collapse>
            </Col>:""}
        
            </Row>
    </Panel>
</Collapse>
    </Col>:""}
    {this.detailsModel.tabpanel1.visible?<Col style={{}}  lg={{ span: 24, offset: 0}}>
        <Tabs    className="">
</Tabs>

    </Col>:""}
    {this.detailsModel.formitem2.visible?<Col style={{}}  lg={{ span: 24, offset: 0}}>
        <div className={`app-form-item app-form-item-left`}>
    {this.detailsModel.formitem2.isShowCaption && !this.detailsModel.formitem2.isEmptyCaption ?
        <div className="app-form-item-label" style={{ width: '130px' }}>
            {this.detailsModel.formitem2.required ? <span className="isrequired">*</span> : ""}
        </div> : ""}
    <div className="app-form-item-container" style={{ width: "calc(100% - 130px)" }}>
        <Form.Item    key={'formitem2'}  validateStatus={this.detailsModel.formitem2.validateStatus} help={this.detailsModel.formitem2.error}>
        <AppDropDownList value={this.data.formitem2} disabled={this.detailsModel.formitem2.disabled}    codelist={this.codelistModel}  placeholder={'请选择...'}  onChange={($event) =>{this.formItemChange('formitem2', $event)}}></AppDropDownList>
        </Form.Item>
    </div>
</div> 

    </Col>:""}
    {this.detailsModel.formitem1.visible?<Col style={{}}  lg={{ span: 24, offset: 0}}>
        <div className={`app-form-item app-form-item-left`}>
    {this.detailsModel.formitem1.isShowCaption && !this.detailsModel.formitem1.isEmptyCaption ?
        <div className="app-form-item-label" style={{ width: '130px' }}>
            {this.detailsModel.formitem1.required ? <span className="isrequired">*</span> : ""}
        </div> : ""}
    <div className="app-form-item-container" style={{ width: "calc(100% - 130px)" }}>
        <Form.Item    key={'formitem1'}  validateStatus={this.detailsModel.formitem1.validateStatus} help={this.detailsModel.formitem1.error}>
        <AppCheckBox value={this.data.formitem1} disabled={this.detailsModel.formitem1.disabled}  onChange={($event) =>{this.formItemChange('formitem1', $event)}}></AppCheckBox>
        </Form.Item>
    </div>
</div> 

    </Col>:""}
    {this.detailsModel.formitem3.visible?<Col style={{}}  lg={{ span: 24, offset: 0}}>
        <div className={`app-form-item app-form-item-left`}>
    {this.detailsModel.formitem3.isShowCaption && !this.detailsModel.formitem3.isEmptyCaption ?
        <div className="app-form-item-label" style={{ width: '130px' }}>
            {this.detailsModel.formitem3.required ? <span className="isrequired">*</span> : ""}
        </div> : ""}
    <div className="app-form-item-container" style={{ width: "calc(100% - 130px)" }}>
        <Form.Item    key={'formitem3'}  validateStatus={this.detailsModel.formitem3.validateStatus} help={this.detailsModel.formitem3.error}>
        <AppDatePicker value={this.data.formitem3} disabled={this.detailsModel.formitem3.disabled}  showTime={true}  format={'YYYY-MM-DD HH:mm:ss'}  placeholder={"请选择时间..."}  onChange={($event) =>{this.formItemChange('formitem3',$event)}}/>
        </Form.Item>
    </div>
</div> 

    </Col>:""}
    {this.detailsModel.formitem4.visible?<Col style={{}}  lg={{ span: 24, offset: 0}}>
        <div className={`app-form-item app-form-item-left`}>
    {this.detailsModel.formitem4.isShowCaption && !this.detailsModel.formitem4.isEmptyCaption ?
        <div className="app-form-item-label" style={{ width: '130px' }}>
            {this.detailsModel.formitem4.required ? <span className="isrequired">*</span> : ""}
        </div> : ""}
    <div className="app-form-item-container" style={{ width: "calc(100% - 130px)" }}>
        <Form.Item    key={'formitem4'}  validateStatus={this.detailsModel.formitem4.validateStatus} help={this.detailsModel.formitem4.error}>
        <AppFileUpload value={this.data.formitem4} disabled={this.detailsModel.formitem4.disabled}  uploadparams={''} exportparams={''} customparams={{}}  onChange={($event) => { this.formItemChange('formitem4', $event) }}></AppFileUpload>
        </Form.Item>
    </div>
</div> 

    </Col>:""}


                </Form>
            </div>
        </>;
    }

    /**
     * 变更监测
     *
     * @protected
     * @memberof Main
     */
    protected tick(): void {
        this.setState({
            date: new Date()
        });
    }

}